from __future__ import annotations

import gi

gi.require_version("Gtk", "3.0")
from gi.repository import Gtk, GLib

from ..engine import Engine, EngineThread
from ..file import RegularFile, FileStatus
from ..utils.decorators import exported_filetype, with_pango_docs

import logging
from tempfile import TemporaryDirectory
from pathlib import Path, PurePath
import os
from time import sleep, time

logger = logging.getLogger(__name__)

SIZE_UNITS = {
    "B": 1,
    "KB": 1000,
    "MB": 1000000,
    "GB": 1000000000,
}


@with_pango_docs(filename="temporary_file_engine.pango")
@exported_filetype(filetype=RegularFile)
class TemporaryFileEngine(Engine):

    NAME = "Temporary File Generator"
    DEBUG = 1

    def __init__(self, appwindow):
        super().__init__(appwindow, FileGeneratorThread, "")

        # Set filesize
        filesize_grid = Gtk.Grid(
            halign=Gtk.Align.FILL,
            valign=Gtk.Align.CENTER,
            hexpand=True,
            vexpand=False,
            column_spacing=5,
        )
        self.attach(filesize_grid, 0, 0, 1, 1)
        label = Gtk.Label(
            label="Filesize: ",
            halign=Gtk.Align.START,
            valign=Gtk.Align.CENTER,
            hexpand=False,
            vexpand=False,
        )
        filesize_grid.attach(label, 1, 0, 1, 1)
        filesize_number_spinbutton = self.register_widget(
            Gtk.SpinButton(
                adjustment=Gtk.Adjustment(
                    lower=1, upper=1024, value=10, page_size=0, step_increment=1
                ),
                value=10,
                update_policy=Gtk.SpinButtonUpdatePolicy.IF_VALID,
                numeric=True,
                climb_rate=5,
                halign=Gtk.Align.START,
                valign=Gtk.Align.CENTER,
                hexpand=False,
                vexpand=False,
            ),
            "filesize_number",
            desensitized=True,
        )
        filesize_grid.attach(filesize_number_spinbutton, 2, 0, 1, 1)
        filesize_unit_combobox = Gtk.ComboBoxText(
            halign=Gtk.Align.START,
            valign=Gtk.Align.CENTER,
            hexpand=False,
            vexpand=False,
        )
        for unit in SIZE_UNITS:
            filesize_unit_combobox.append_text(unit)
        filesize_unit_combobox.set_active(0)
        self.register_widget(
            filesize_unit_combobox, "filesize_unit", desensitized=True
        )
        filesize_grid.attach(filesize_unit_combobox, 3, 0, 1, 1)

        # Add horizontal separator
        separator = Gtk.Separator(
            orientation=Gtk.Orientation.HORIZONTAL,
            halign=Gtk.Align.FILL,
            valign=Gtk.Align.CENTER,
            hexpand=True,
            vexpand=False,
        )
        self.attach(separator, 0, 1, 3, 1)

        # set time between files being created
        time_grid = Gtk.Grid(
            halign=Gtk.Align.FILL,
            valign=Gtk.Align.CENTER,
            hexpand=True,
            vexpand=False,
            column_spacing=5,
        )
        self.attach(time_grid, 0, 2, 1, 1)
        label = Gtk.Label(
            label="Time between file creation events: ",
            halign=Gtk.Align.START,
            valign=Gtk.Align.CENTER,
            hexpand=False,
            vexpand=False,
        )
        time_grid.attach(label, 1, 0, 1, 1)
        time_number_spinbutton = self.register_widget(
            Gtk.SpinButton(
                adjustment=Gtk.Adjustment(
                    lower=1,
                    upper=3600 * 24,
                    value=5,
                    page_size=0,
                    step_increment=1,
                ),
                value=5,
                update_policy=Gtk.SpinButtonUpdatePolicy.IF_VALID,
                numeric=True,
                climb_rate=5,
                halign=Gtk.Align.START,
                valign=Gtk.Align.CENTER,
                hexpand=False,
                vexpand=False,
            ),
            "creation_delay",
            desensitized=True,
        )
        time_grid.attach(time_number_spinbutton, 2, 0, 1, 1)

        # Add vertical separator
        separator = Gtk.Separator(
            orientation=Gtk.Orientation.VERTICAL,
            halign=Gtk.Align.CENTER,
            valign=Gtk.Align.FILL,
            hexpand=False,
            vexpand=True,
        )
        self.attach(separator, 1, 0, 1, 3)

        # start index
        start_index_grid = Gtk.Grid(
            halign=Gtk.Align.FILL,
            valign=Gtk.Align.CENTER,
            hexpand=True,
            vexpand=False,
            column_spacing=5,
        )
        self.attach(start_index_grid, 2, 0, 1, 1)
        label = Gtk.Label(
            label="Start index: ",
            halign=Gtk.Align.START,
            valign=Gtk.Align.CENTER,
            hexpand=False,
            vexpand=False,
        )
        start_index_grid.attach(label, 0, 0, 1, 1)
        start_index_spinbutton = self.register_widget(
            Gtk.SpinButton(
                adjustment=Gtk.Adjustment(
                    lower=0, upper=10000, value=0, page_size=0, step_increment=1
                ),
                value=0,
                update_policy=Gtk.SpinButtonUpdatePolicy.IF_VALID,
                numeric=True,
                climb_rate=5,
                halign=Gtk.Align.START,
                valign=Gtk.Align.CENTER,
                hexpand=False,
                vexpand=False,
            ),
            "start_index",
            desensitized=False,
        )
        start_index_grid.attach(start_index_spinbutton, 1, 0, 1, 1)

        # prefix
        prefix_grid = Gtk.Grid(
            halign=Gtk.Align.FILL,
            valign=Gtk.Align.CENTER,
            hexpand=True,
            vexpand=False,
            column_spacing=5,
        )
        self.attach(prefix_grid, 2, 2, 1, 1)
        label = Gtk.Label(
            label="File prefix: ",
            halign=Gtk.Align.START,
            valign=Gtk.Align.CENTER,
            hexpand=False,
            vexpand=False,
        )
        prefix_grid.attach(label, 0, 0, 1, 1)
        self._prefix_entry = self.register_widget(
            Gtk.Entry(
                text="test_",
                halign=Gtk.Align.START,
                valign=Gtk.Align.CENTER,
                hexpand=False,
                vexpand=False,
            ),
            "file_prefix",
            desensitized=False,
        )
        prefix_grid.attach(self._prefix_entry, 1, 0, 1, 1)

        # this starts out as valid
        self._valid = True

    def _file_prefix_entry_changed_cb(self, entry):
        if self.params.file_prefix:
            self._valid = True
        else:
            self._valid = False

        self.notify("valid")


class FileGeneratorThread(EngineThread):

    SUFFIX = ".dat"

    def run(self):
        # close task_window
        GLib.idle_add(
            self._engine.kill_task_window,
            self._task_window,
            priority=GLib.PRIORITY_HIGH,
        )

        # sleep for 1 sec to not have the task window flash
        sleep(1)

        index = int(self.params.start_index)
        with TemporaryDirectory() as tempdir:
            while 1:
                if self.should_exit:
                    self._engine.cleanup()
                    return
                basename = f"{self.params.file_prefix}{index}{self.SUFFIX}"
                path = Path(tempdir, basename)
                path.write_bytes(
                    os.urandom(
                        int(
                            self.params.filesize_number
                            * SIZE_UNITS[self.params.filesize_unit]
                        )
                    )
                )
                index = index + 1
                if (
                    self._engine.props.running
                    and self._engine._appwindow._queue_manager.props.running
                ):
                    _file = RegularFile(
                        str(path),
                        PurePath(basename),
                        time(),
                        FileStatus.CREATED,
                    )
                    GLib.idle_add(
                        self._engine._appwindow._queue_manager.add,
                        _file,
                        priority=GLib.PRIORITY_HIGH,
                    )
                # ensure we dont need no to wait too long when stopping the engine
                for _ in range(int(self._engine.params.creation_delay)):
                    sleep(1)
                    if self.should_exit:
                        self._engine.cleanup()
                        return



import gi

gi.require_version("Gtk", "3.0")
from gi.repository import Gtk

import logging

logger = logging.getLogger(__name__)


class ParamsWindow(Gtk.Window):
    def __init__(
        self, contents: Gtk.Widget, parent: Gtk.ApplicationWindow, title: str
    ):
        super().__init__(
            destroy_with_parent=True,
            transient_for=parent,
            window_position=Gtk.WindowPosition.NONE,
            border_width=5,
            title=title,
        )
        self.add(contents)
        contents.show_all()

    def do_delete_event(self, event):
        return self.hide_on_delete()



from threading import current_thread
import gi

gi.require_version("Gtk", "3.0")
from gi.repository import Gtk

from ..operation import Operation
from ..file import Directory
from ..utils import ExitableThread, get_random_string
from ..utils.decorators import supported_filetypes, with_pango_docs
from ..utils.exceptions import SkippedOperation

import logging
from dataclasses import dataclass, field
from typing import Callable, List, Dict, Any, Tuple
import tarfile
import zipfile
from pathlib import Path
import os.path


logger = logging.getLogger(__name__)


@dataclass
class Compressor:
    opener: Callable
    mode: str
    adder: str  # 'add' (tar) or 'write' (zip)
    suffix: str
    opener_args: Dict[str, Any] = field(default_factory=dict)


COMPRESSORS: Dict[str, Compressor] = {
    "TAR + GZIP": Compressor(tarfile.open, "w:gz", "add", ".tar.gz"),
    "TAR + BZIP2": Compressor(tarfile.open, "w:bz2", "add", ".tar.bz2"),
    "TAR + LZMA": Compressor(tarfile.open, "w:xz", "add", ".tar.xz"),
    "TAR": Compressor(tarfile.open, "w", "add", ".tar"),
    "ZIP": Compressor(
        zipfile.ZipFile,
        "w",
        "write",
        ".zip",
        opener_args=dict(compression=zipfile.ZIP_DEFLATED),
    ),
}


@supported_filetypes(filetypes=Directory)
@with_pango_docs(filename="directory_compressor.pango")
class DirectoryCompressorOperation(Operation):

    NAME = "Directory Compressor"

    def __init__(self, *args, **kwargs):
        Operation.__init__(self, *args, **kwargs)
        grid = Gtk.Grid(
            border_width=5,
            row_spacing=5,
            column_spacing=5,
            halign=Gtk.Align.FILL,
            valign=Gtk.Align.CENTER,
            hexpand=True,
            vexpand=False,
        )
        self.add(grid)

        # boxes are needed for
        # destination folder
        # compression type

        label = Gtk.Label(
            label="Destination",
            halign=Gtk.Align.START,
            valign=Gtk.Align.CENTER,
            hexpand=False,
            vexpand=False,
        )
        grid.attach(label, 0, 0, 1, 1)

        directory_chooser_button = self.register_widget(
            Gtk.FileChooserButton(
                title="Select a directory to copy compressed files to",
                action=Gtk.FileChooserAction.SELECT_FOLDER,
                create_folders=True,
                halign=Gtk.Align.FILL,
                valign=Gtk.Align.CENTER,
                hexpand=True,
                vexpand=False,
            ),
            "destination_directory",
        )
        grid.attach(directory_chooser_button, 1, 0, 1, 1)

        grid.attach(
            Gtk.Separator(
                orientation=Gtk.Orientation.HORIZONTAL,
                halign=Gtk.Align.FILL,
                valign=Gtk.Align.CENTER,
                hexpand=True,
                vexpand=False,
            ),
            0,
            1,
            2,
            1,
        )

        label = Gtk.Label(
            label="Compression Type",
            halign=Gtk.Align.START,
            valign=Gtk.Align.CENTER,
            hexpand=False,
            vexpand=False,
        )
        grid.attach(label, 0, 2, 1, 1)

        compression_type_combo_box = self.register_widget(
            Gtk.ComboBoxText(
                halign=Gtk.Align.START,
                valign=Gtk.Align.CENTER,
                hexpand=False,
                vexpand=False,
            ),
            "compression_type",
        )

        grid.attach(compression_type_combo_box, 1, 2, 1, 1)
        for compression_type in COMPRESSORS:
            compression_type_combo_box.append_text(compression_type)
        compression_type_combo_box.set_active(0)

    def preflight_check(self):
        # ensure destination is not None
        if self.params.destination_directory is None:
            raise ValueError("Destination folder cannot be empty")

        # ensure we are not writing into the monitored directory
        from ..engines.directory_watchdog_engine import DirectoryWatchdogEngine

        if not isinstance(
            self.appwindow.active_engine, DirectoryWatchdogEngine
        ):
            raise ValueError(
                "DirectoryCompressor currently only works with DirectoryWatchdog engines!"
            )

        self._monitored_directory = (
            self.appwindow.active_engine._get_params().monitored_directory
        )

        if Path(self.params.destination_directory).samefile(
            self._monitored_directory
        ):
            raise ValueError(
                "Destination folder cannot be the same as the monitored directory"
            )

        try:
            Path(self.params.destination_directory).resolve().relative_to(
                Path(self._monitored_directory)
            )
        except ValueError:
            pass
        else:
            raise ValueError(
                "The destination directory cannot be a subdirectory of the monitored directory."
            )

        # ensure the destination is writable
        tempfile = Path(
            self.params.destination_directory, get_random_string(10)
        )
        try:
            with tempfile.open("w") as f:
                f.write("teststring")
        except Exception as e:
            raise ValueError(f"Cannot write to destination folder: {str(e)}")
        else:
            tempfile.unlink(missing_ok=True)

    def _check_existing_zipfile(
        self, _zipfile: Path, file_list: List[Tuple[str, int]]
    ):
        if not _zipfile.exists() or not zipfile.is_zipfile(_zipfile):
            return

        with zipfile.ZipFile(_zipfile) as f:
            zipped_files = dict(
                zip(
                    map(
                        lambda x: os.path.join(self._monitored_directory, x),
                        f.namelist(),
                    ),
                    f.infolist(),
                )
            )

        if len(zipped_files) != len(file_list):
            return

        for _filename, _size in file_list:
            if _filename not in zipped_files:
                return
            zipped_file = zipped_files[_filename]
            if _size != zipped_file.file_size:
                return
            elif os.stat(_filename).st_mtime > _zipfile.stat().st_mtime:
                return

        raise SkippedOperation("Zipfile contents are equal to directory")

    def _check_existing_tarfile(
        self, _tarfile: Path, file_list: List[Tuple[str, int]]
    ):
        if not _tarfile.exists() or not tarfile.is_tarfile(_tarfile):
            return

        with tarfile.open(_tarfile) as f:
            zipped_files = dict(
                zip(
                    map(
                        lambda x: os.path.join(self._monitored_directory, x),
                        f.getnames(),
                    ),
                    f.getmembers(),
                )
            )

        if len(zipped_files) != len(file_list):
            return

        for _filename, _size in file_list:
            if _filename not in zipped_files:
                return
            zipped_file = zipped_files[_filename]
            if _size != zipped_file.size:
                return
            elif os.stat(_filename).st_mtime > zipped_file.mtime:
                return

        raise SkippedOperation("Tarball contents are equal to directory")

    def run(self, dir: Directory):  # type: ignore[override]

        compressor = COMPRESSORS[self.params.compression_type]

        destination_filename = Path(
            self.params.destination_directory, dir.relative_filename.name
        ).with_suffix(compressor.suffix)

        our_thread = current_thread()

        total_size = dir.total_size
        file_list = list(dir)
        number_of_files = len(dir)
        size_seen = 0

        if "tar" in compressor.suffix:
            self._check_existing_tarfile(destination_filename, file_list)
        elif "zip" in compressor.suffix:
            self._check_existing_zipfile(destination_filename, file_list)

        with compressor.opener(
            destination_filename, compressor.mode, **compressor.opener_args
        ) as f:
            for _file_index, (_file, _size) in enumerate(dir):
                if (
                    isinstance(our_thread, ExitableThread)
                    and our_thread.should_exit
                ):
                    return "Job aborted"
                arcname = os.path.relpath(_file, self._monitored_directory)
                getattr(f, compressor.adder)(_file, arcname)

                if total_size == 0:
                    dir.update_progressbar(
                        self.index, 100.0 * (_file_index + 1) / number_of_files
                    )
                else:
                    size_seen += _size
                    dir.update_progressbar(
                        self.index, 100.0 * size_seen / total_size
                    )



from __future__ import annotations
import gi

gi.require_version("Gtk", "3.0")
from gi.repository import Gtk, GLib
import boto3
from botocore.client import Config
import pika

import logging
import urllib
import ssl
from pathlib import Path
import time
import json

from .base_s3_bucket_engine import BaseS3BucketEngine, BaseS3BucketEngineThread
from .ceph_s3_bucket_engine_advanced_settings import (
    CephS3BucketEngineAdvancedSettings,
)
from ..file import S3Object
from ..utils.decorators import (
    exported_filetype,
    with_advanced_settings,
    with_pango_docs,
)
from ..utils import get_random_string

logger = logging.getLogger(__name__)

ABORT_MESSAGE = "Ensure you are using valid credentials and bucket name, and that a push-endpoint has been properly configured in Advanced Settings."


@exported_filetype(filetype=S3Object)
@with_advanced_settings(
    engine_advanced_settings=CephS3BucketEngineAdvancedSettings
)
@with_pango_docs(filename="ceph_s3_bucket_engine.pango")
class CephS3BucketEngine(BaseS3BucketEngine):

    NAME = "Ceph S3 Bucket Monitor"

    def __init__(self, appwindow):
        super().__init__(appwindow, CephS3BucketEngineThread, ABORT_MESSAGE)

        # Needs:
        # 1. endpoint
        # 2. bucket name
        # 3. access key
        # 4. secret key

        # the rest of the config goes into advanced
        # add endpoint
        self.attach(
            Gtk.Label(
                label="Ceph Endpoint",
                halign=Gtk.Align.START,
                valign=Gtk.Align.CENTER,
                hexpand=False,
                vexpand=False,
            ),
            0,
            0,
            1,
            1,
        )
        self._ceph_endpoint_entry = self.register_widget(
            Gtk.Entry(
                halign=Gtk.Align.FILL,
                valign=Gtk.Align.CENTER,
                hexpand=True,
                vexpand=False,
            ),
            "ceph_endpoint",
        )
        self.attach(self._ceph_endpoint_entry, 1, 0, 1, 1)

        # add bucket name -> this bucket must already exist!!!
        self.attach(
            Gtk.Label(
                label="Bucket Name",
                halign=Gtk.Align.START,
                valign=Gtk.Align.CENTER,
                hexpand=False,
                vexpand=False,
            ),
            2,
            0,
            1,
            1,
        )
        self._bucket_name_entry = self.register_widget(
            Gtk.Entry(
                halign=Gtk.Align.FILL,
                valign=Gtk.Align.CENTER,
                hexpand=True,
                vexpand=False,
            ),
            "bucket_name",
        )
        self.attach(self._bucket_name_entry, 3, 0, 1, 1)

        # Access key
        self.attach(
            Gtk.Label(
                label="Access Key",
                halign=Gtk.Align.START,
                valign=Gtk.Align.CENTER,
                hexpand=False,
                vexpand=False,
            ),
            0,
            1,
            1,
            1,
        )
        self._access_key_entry = self.register_widget(
            Gtk.Entry(
                halign=Gtk.Align.FILL,
                valign=Gtk.Align.CENTER,
                hexpand=True,
                vexpand=False,
            ),
            "access_key",
        )
        self.attach(self._access_key_entry, 1, 1, 1, 1)

        # Secret key
        self.attach(
            Gtk.Label(
                label="Secret Key",
                halign=Gtk.Align.START,
                valign=Gtk.Align.CENTER,
                hexpand=False,
                vexpand=False,
            ),
            2,
            1,
            1,
            1,
        )
        self._secret_key_entry = self.register_widget(
            Gtk.Entry(
                visibility=False,
                halign=Gtk.Align.FILL,
                valign=Gtk.Align.CENTER,
                hexpand=True,
                vexpand=False,
            ),
            "secret_key",
            exportable=False,
        )
        self.attach(self._secret_key_entry, 3, 1, 1, 1)

        self.attach(
            Gtk.Label(
                label="<b>Please use Advanced Settings to configure the push-endpoint</b>",
                halign=Gtk.Align.FILL,
                valign=Gtk.Align.CENTER,
                hexpand=True,
                vexpand=False,
                xalign=0.5,
                use_markup=True,
            ),
            0,
            2,
            4,
            1,
        )

        # connect signal handlers to determine validity
        self._ceph_endpoint_entry.connect("changed", self._s3_entry_changed_cb)
        self._access_key_entry.connect("changed", self._s3_entry_changed_cb)
        self._secret_key_entry.connect("changed", self._s3_entry_changed_cb)
        self._bucket_name_entry.connect("changed", self._s3_entry_changed_cb)

    def _s3_entry_changed_cb(self, entry):
        # todo: implement better bucket name validation
        if (
            self.params.ceph_endpoint.startswith("https://")
            and self.params.bucket_name
            and self.params.access_key
            and self.params.secret_key
        ):
            self._valid = True
        else:
            self._valid = False

        self.notify("valid")

    def _get_client_options(self) -> dict:
        rv = super()._get_client_options()
        rv["endpoint_url"] = self._get_params().ceph_endpoint
        return rv

    def cleanup(self):
        # delete notification config
        try:
            self.s3_client.delete_bucket_notification_configuration(
                Bucket=self._get_params().bucket_name
            )
        except Exception as e:
            logger.exception(
                f"Could not restore bucket notification config: {str(e)}"
            )

        # remove topic
        if hasattr(self, "topic_arn"):
            try:
                self.sns_client.delete_topic(TopicArn=self.topic_arn)
            except Exception as e:
                logger.exception(
                    f"Could not remove topic {self.topic_arn}: {str(e)}"
                )

        super().cleanup()


class CephS3BucketEngineThread(BaseS3BucketEngineThread):
    def get_full_name(self, key):
        return f"{self.params.ceph_endpoint}/{self.params.bucket_name}/{key}"

    def run(self):
        self._client_options = self._engine._get_client_options()

        if not self.get_region_name(""):
            return

        # set up proper s3 client
        self._engine.s3_client = boto3.client("s3", **self._client_options)

        # set up sns client
        self._engine.sns_client = boto3.client(
            "sns", **self._client_options, config=Config(signature_version="s3")
        )

        try:
            # get current bucket notifications
            # response = self._engine.s3_client.get_bucket_notification_configuration(
            #    Bucket=self.params.bucket_name,
            # )

            # self._engine.old_bucket_notification_config = {configs: response.get(configs, []) for configs in AVAILABLE_CONFIGURATIONS}

            GLib.idle_add(
                self._task_window.set_text,
                "<b>Configuring bucket notifications...</b>",
            )
            self._engine.s3_client.delete_bucket_notification_configuration(
                Bucket=self.params.bucket_name,
            )

            if not self.params.rabbitmq_vhost.startswith("/"):
                raise ValueError("RabbitMQ vhost must start with a /")
            elif self.params.rabbitmq_vhost == "/":
                vhost = ""
            else:
                vhost = self.params.rabbitmq_vhost

            endpoint_args = (
                f"push-endpoint=amqp://{self.params.rabbitmq_username}:"
                + f"{self.params.rabbitmq_password}@{self.params.rabbitmq_hostname}:"
                + f"{int(self.params.rabbitmq_producer_port)}{vhost}&"
                + f"amqp-exchange={self.params.rabbitmq_exchange}&"
                + f"amqp-ack-level=broker"
            )
            attributes = {
                nvp[0]: nvp[1]
                for nvp in urllib.parse.parse_qsl(
                    endpoint_args, keep_blank_values=True
                )
            }

            topic_name = f"rfi-file-monitor-ceph.{get_random_string(8)}"

            resp = self._engine.sns_client.create_topic(
                Name=topic_name, Attributes=attributes
            )
            self._engine.topic_arn = resp["TopicArn"]

            topic_conf_list = [
                {
                    "TopicArn": self._engine.topic_arn,
                    "Events": ["s3:ObjectCreated:*", "s3:ObjectRemoved:*"],
                    "Id": "another-one-bites-the-dust",
                },
            ]

            self._engine.s3_client.put_bucket_notification_configuration(
                Bucket=self.params.bucket_name,
                NotificationConfiguration={
                    "TopicConfigurations": topic_conf_list
                },
            )

            # test connection to rabbitmq
            GLib.idle_add(
                self._task_window.set_text, "<b>Testing AMQP server...</b>"
            )
            if self.params.rabbitmq_consumer_use_ssl:
                ssl_context = ssl.create_default_context()
                if (
                    self.params.rabbitmq_ca_certificate
                    and (
                        ca_cert := Path(self.params.rabbitmq_ca_certificate)
                    ).exists()
                ):
                    ssl_context.load_verify_locations(cafile=ca_cert)
                ssl_options = pika.SSLOptions(ssl_context)
            else:
                ssl_options = None
            credentials = pika.credentials.PlainCredentials(
                self.params.rabbitmq_username, self.params.rabbitmq_password
            )
            connection_params = pika.ConnectionParameters(
                host=self.params.rabbitmq_hostname,
                port=int(self.params.rabbitmq_consumer_port),
                virtual_host=self.params.rabbitmq_vhost,
                credentials=credentials,
                ssl_options=ssl_options,
            )

            with pika.BlockingConnection(connection_params) as connection:
                with connection.channel() as channel:
                    channel.exchange_declare(
                        exchange=self.params.rabbitmq_exchange,
                        exchange_type="topic",
                        durable=True,
                    )
        except Exception as e:
            self._engine.cleanup()
            GLib.idle_add(
                self._engine.abort,
                self._task_window,
                e,
                priority=GLib.PRIORITY_HIGH,
            )
            return

        # sleep 1 second
        time.sleep(1)

        # if required, add existing files to queue
        if (
            self.params.process_existing_files
            and not self.process_existing_files()
        ):
            return

        # if we get here, things should be working.
        # close task_window
        GLib.idle_add(
            self._engine.kill_task_window,
            self._task_window,
            priority=GLib.PRIORITY_HIGH,
        )

        # start the big while loop and start consuming incoming messages
        try:
            with pika.BlockingConnection(connection_params) as connection:
                with connection.channel() as channel:
                    channel.exchange_declare(
                        exchange=self.params.rabbitmq_exchange,
                        exchange_type="topic",
                        durable=True,
                    )
                    result = channel.queue_declare("", exclusive=True)
                    queue_name = result.method.queue
                    channel.queue_bind(
                        exchange=self.params.rabbitmq_exchange,
                        queue=queue_name,
                        routing_key=topic_name,
                    )

                    while True:
                        if self._should_exit:
                            self._engine.cleanup()
                            return

                        method_frame, _, _body = channel.basic_get(queue_name)
                        if method_frame:
                            body = json.loads(_body)
                            channel.basic_ack(method_frame.delivery_tag)
                            # we are going to assume 1 record per message
                            try:
                                record = body["Records"][0]
                                event_name: str = record["eventName"]
                            except Exception as e:
                                logger.info(
                                    f"Ignoring {_body=} because of {str(e)}"
                                )
                                continue

                            if "ObjectCreated" in event_name:
                                # new file created!
                                s3_info = record["s3"]
                                if not self.process_new_file(s3_info):
                                    continue
                        else:
                            time.sleep(1)
        except Exception as e:
            self._engine.cleanup()
            GLib.idle_add(
                self._engine.abort, None, e, priority=GLib.PRIORITY_HIGH
            )
            return
